import {
  Modal, ModalService, Button, SelectMultiple, Form, DialogService, LoadingService,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, IEventParam, Loader, Http, I18n,
} from '@zeedhi/core';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pme04001AjustarComensaisEspeciais {
  private modalAjuste!: Modal;
  
  private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
  
  public appController = Loader.getInstance('AppController');
  
  public utilities = Loader.getInstance('Utilities');
    
  public showModal({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    this.validAndOpenModal();
  }
  
  public async createModal() {
    this.modalAjuste = await ModalService.createFromJsonPost('',
      { jsonPath: 'pme/cardapio/modals/pme04001_ajustar_comensais_especiais' });
  }
  
  public async validAndOpenModal() {
    if (this.elaboracaoCardapio.getFilterData.NRCARDPME) {
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      cardapio.setValueAction(true);
      if (this.utilities.checkOpenModalOverlap()) {
        if (!this.modalAjuste) {
          this.createModal().then(() => {
            this.modalAjuste.show();
          });
        } else {
          this.modalAjuste.show();
        }
      }
    } else {
      this.utilities.showAlert('withoutFilter', 'Operação bloqueada! Favor realizar o filtro.', 'error');
    }
  }
  
  public cancelModal() {
    if (this.modalAjuste) {
      this.modalAjuste.destroy();
      this.modalAjuste = this.utilities.modalEmpty;
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      cardapio.setValueAction(false);
    }
  }
  
  public filterSelectMultiple({ component }: IEventParam<SelectMultiple>) {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: cardapio.filterData.cdfilcentpme,
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: 'T',
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cardapio.filterData.typeMenu.CDTIPOCARD,
      },
      {
        name: 'CDSERVFILI',
        operator: '=',
        value: cardapio.filterData.service.CDSERVFILI,
      },
    ];
      
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public resetDate({ component }: IEventParam<IDictionary>) {
    component.value = null;
    const finalDate: IDictionary = Metadata.getInstance('finalDateAjust');
    finalDate.value = null;
  }
  
  public setInitialDate({ component }: IEventParam<IDictionary>) {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    const date = cardapio.getCurrentSelectedRegister()?.DTCARDIAPME.substring(0, 10);
    if (component.name === 'initialDatejust') {
      component.value = date;
    } else {
      component.value = date;
    }
  }
  
  public confirmAjuste() {
    const form = Metadata.getInstance<Form>('formAjust');
    if (form.validate()) {
      DialogService.show({
        name: 'excludePeriodoCardapio',
        type: 'warning',
        title: 'Atenção',
        text: 'pme::ajustarComenEspecPeriodo',
        buttons: [
          {
            name: 'yesButtonAjust',
            component: 'ZdButton',
            label: 'Sim',
            events: { click: () => this.callAjusteComensaisEspeciais() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'Não',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public callAjusteComensaisEspeciais() {
    const form: Form = Metadata.getInstance('formAjust');
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
      
    if (form.validate()) {
      const initialDate = cardapio.dates.find(
        (date: IDictionary) => date.DTCARDIAPME.substring(0, 10) === form.value.initialDateAjust,
      );
      const finalDate = cardapio.dates.find(
        (date: IDictionary) => date.DTCARDIAPME.substring(0, 10) === form.value.finalDateAjust,
      );
      if (initialDate !== undefined && finalDate !== undefined) {
        if (!this.bloqDayInPeriod(form.value.initialDateAjust, form.value.finalDateAjust, cardapio)) {
          const row = {
            CDFILCENTPME: this.appController.getCdCentral(),
            CDTIPOCARD: cardapio.filterData.typeMenu.CDTIPOCARD,
            CDSERVFILI: cardapio.filterData.service.CDSERVFILI,
            NRCARDPME: this.elaboracaoCardapio.getFilterData.NRCARDPME,
            ...form.value,
          };
          LoadingService.show();
          Http.post('/procedureAjustComEspPrat', { requestType: 'Row', row }).then((response: IDictionary) => {
            // const nrLogIntCard = response?.data?.dataset?.result?.P_NRLOGINTCARD;
            if (response?.data?.dataset?.result?.length === 0) {
              this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
              this.cancelModal();
              this.elaboracaoCardapio.reloadCard();
            } else {
              // tradução impeditivos
              // mapeamento de P_NRLOGINTCARD com base na procedure
              let errorMessage = '';
              response?.data?.dataset?.result?.forEach((error: any) => {
                const params = Object.values(error)?.filter((el: any) => el !== null && el !== ' | ')?.join(' - ') || '<br>';
                switch (error?.NRERRO) {
                  case 0:
                    errorMessage += `${I18n.translate('pme::pratoDestInativo', { PARAM1: params })} <br>`;
                    break;
                  case 1:
                    errorMessage += `${I18n.translate('pme::pratoDestSemReceita', { PARAM1: params })} <br>`;
                    break;
                  case 2:
                    errorMessage += `${I18n.translate('pme::pratoDestSemReceitaAtiva', { PARAM1: params })} <br>`;
                    break;
                  case 3:
                    errorMessage += `${I18n.translate('pme::pratoDestSemReceitaRevisada', { PARAM1: params })} <br>`;
                    break;
                  case 4:
                    errorMessage += `${I18n.translate('pme::opBloqProdutoInativoReceita', { PARAM1: params })} <br>`;
                    break;
                  case 5:
                    errorMessage += `${I18n.translate('pme::dtProcessamentoForaPeriodo', { PARAM1: params })} <br>`;
                    break;
                  default:
                    errorMessage += `${I18n.translate('pme::excecao', { PARAM1: params })} <br>`;
                }
              });
              DialogService.show({
                name: 'errorResponse',
                type: 'error',
                title: 'pme::atencao',
                text: `${I18n.translate('pme::erroAjuste')}: <br>${errorMessage}`,
                buttons: [
                  {
                    name: 'ok',
                    component: 'ZdButton',
                    label: 'pme::OK',
                    events: { click: () => DialogService.hide() },
                  },
                ],
              });
            }
          }).catch((error: IDictionary) => {
            this.utilities.showAlert('errorResponse', error, 'error');
          }).finally(() => {
            DialogService.hide();
            LoadingService.hide();
          });
        } else {
          this.utilities.showAlert('dateIsLessTheDateOfProcess',
            'pme::foraDoPeriodoDeProcessamentoCard',
            'error');
          DialogService.hide();
        }
      } else {
        this.utilities.showAlert('outOfPeriod', 'pme::dataForaDoPeriodo', 'error');
        DialogService.hide();
      }
    }
  }
  
  private bloqDayInPeriod(initialDate: string, finalDate: string, cardapio: Cardapio): boolean {
    let hasBloqDay = false;
    cardapio.dates.forEach((date: IDictionary) => {
      if (date.DTCARDIAPME.substring(0, 10) >= initialDate && date.DTCARDIAPME.substring(0, 10) <= finalDate) {
        if (!cardapio.dateLessDateOfProcess(date.DTCARDIAPME.substring(0, 10))) hasBloqDay = true;
      }
    });
    return hasBloqDay;
  }
}
